require 'spec_helper'


describe OO::CheckLicense do

  describe '.new(attrs = {})' do
    let(:service_object) { described_class.new(attrs) }
    let(:license_key) { 'glass.take' }

    subject { service_object }

    context 'when required attrs are missing' do
      context 'when :key (checked license key) is missing' do
        let(:attrs) { { in: [] } }
        it { expect { subject }.to raise_error(KeyError) }
      end

      context 'when :in (license keys list) is missing' do
        let(:attrs) { { key: license_key } }
        it { expect { subject }.to raise_error(KeyError) }
      end
    end

    context 'when all required attrs were passed' do
      let(:attrs) { { key: license_key, in: [] } }

      it { is_expected.to be_a(described_class) }

      describe 'builds a license' do
        let(:license_class) { OO::License }

        describe '@license' do
          let(:license) { service_object.instance_variable_get(:@license) }

          subject { license }
          it { is_expected.to be_a(license_class) }

          describe '#key' do
            subject { license.key }
            it { is_expected.to eq(license_key) }
          end
        end
      end
    end
  end

  describe '#call' do
    let(:service_object) { described_class.new(key: license_key, in: license_keys) }
    let(:license) { OO::License.new(key: license_key) }
    let(:license_key) { 'glass.take' }

    subject { service_object.call }

    describe 'returns all keys wider or equal to given key found in given list' do
      context 'when license key is included in license keys list' do
        context 'and is an only match' do
          let(:license_keys) { [license_key] }
          it { is_expected.to eq([license_key]) }
        end
      end

      context 'when exact license key is missing in given list' do
        context 'but there are wider keys' do
          let(:license_keys) { license.genre_masks }
          it { is_expected.to eq(license.genre_masks) }
        end
      end

      context 'when license key is missing in given list' do
        let(:license_keys) { %w(a b c d) }
        it { is_expected.to eq([]) }

        context 'but found parent key' do
          let(:license_keys) { [license.parent.key] }
          it { is_expected.to eq([]) }
        end
      end
    end
  end
end
